%  Traning Feed-forward Neural Networks using Grey Wolf Optimizer   %
%                                                                   %
%  Developed in MATLAB R2011b(7.13)                                 %
%                                                                   %
%  Author and programmer: Seyedali Mirjalili                        %
%                                                                   %
%         e-Mail: ali.mirjalili@gmail.com                           %
%                 seyedali.mirjalili@griffithuni.edu.au             %
%                                                                   %
%       Homepage: http://www.alimirjalili.com                       %
%                                                                   %
%   Main paper: S. Mirjalili,How effective is the Grey Wolf         %
%               optimizer in training multi-layer perceptrons       %
%              Applied Intelligece, in press, 2015,                 %
%               http://dx.doi.org/10.1007/s10489-014-0645-7         %
%                                                                   %

% All the datasets have been included in this version

clear all 
clc

Runno=1;

SearchAgents_no=200; % Number of search agents

% classification datasets

% Function_name='F1'; %MLP_XOR dataset
% Function_name='F2'; %MLP_Baloon dataset
 Function_name='F3'; %MLP_Iris dataset
% Function_name='F4'; %MLP_Cancer dataset
% Function_name='F5'; %MLP_Heart dataset

% Function approximation datasets

% Function_name='F6'; %MLP_Sigmoid dataset
% Function_name='F7'; %MLP_Cosine dataset
% Function_name='F8'; %MLP_Sine dataset
% Function_name='F9'; %MLP_Sphere dataset

% Load details of the selected data set
[lb,ub,dim,fobj]=Get_Functions_details(Function_name);


Max_iteration=200; % Maximum numbef of iterations

[Best_score,Best_pos,GWO_cg_curve]=GWO(SearchAgents_no,Max_iteration,lb,ub,dim,fobj);


semilogy((GWO_cg_curve),'Color','r')
hold on
%semilogy(PSO_cg_curve,'Color','b')
title('Objective space')
xlabel('Iteration');
ylabel('Best score obtained so far');

axis tight
grid on
box on
legend('GWO')

if Function_name=='F1' 
    
input=  [0 0 0 0 1 1 1 1;0 0 1 1 0 0 1 1;0 1 0 1 0 1 0 1];
target3=[0 1 1 0 1 0 0 1];
 Hno=7;
dim = 5*7+1;                      % Dimension of the problem
 
    for i=1:Runno
               Rrate=0;
        W=Best_pos(i,1:4*Hno);
        B=Best_pos(i,4*Hno+1:dim);
 
        for pp=1:8
            actualvalue=my_simulate(3,7,1,W,B,input(:,pp)');
            if(target3(pp)==1)
                if (actualvalue>=0.95)
                    Rrate=Rrate+1;
                end
            end
            if(target3(pp)==0)
                if (actualvalue(1)<0.05)
                    Rrate=Rrate+1;
                end  
            end
        end
         
        Final_rate(i)=(Rrate/8)*100;
    end
    
    ['classification rate = ' , num2str(Final_rate(i))]
end
    
    
if Function_name=='F2'

load baloon.txt
 x=sortrows(baloon,2);
 %I2=x(1:150,1:4);
 I2(:,1)=x(1:20,1);
 I2(:,2)=x(1:20,2);
 I2(:,3)=x(1:20,3);
 I2(:,4)=x(1:20,4);
 T=x(1:20,5);
 

Hno=9;
dim = 6*9+1
 
   for i=1:Runno
        Rrate=0;
        W=Best_pos(i,1:45);
        B=Best_pos(i,46:55);
        for pp=1:20
            actualvalue=my_simulate(4,9,1,W,B,I2(pp,:));
            if(T(pp)==1)
                if (actualvalue>=0.95)
                    Rrate=Rrate+1;
                end
            end
            if(T(pp)==0)
                if (actualvalue(1)<0.05)
                    Rrate=Rrate+1;
                end  
            end

        end
        
        
      Final_rate(i,:)=(Rrate/20)*100;

   end
   ['classification rate = ' , num2str(Final_rate(i))]
  
end

if Function_name=='F3' 
    
    load iris.txt
 x=sortrows(iris,2);
 %I2=x(1:150,1:4);
 H2=x(1:150,1);
 H3=x(1:150,2);
 H4=x(1:150,3);
 H5=x(1:150,4);
 T=x(1:150,5);
 %I=(I2-0.1)./(7.9-0.1)
 H2=H2';
 [xf,PS] = mapminmax(H2);
 I2(:,1)=xf;
 
 H3=H3';
 [xf,PS2] = mapminmax(H3);
 I2(:,2)=xf;
 
 H4=H4';
 [xf,PS3] = mapminmax(H4);
 I2(:,3)=xf;
 
 H5=H5';
 [xf,PS4] = mapminmax(H5);
 I2(:,4)=xf;
 Thelp=T;
 T=T';
 [yf,PS5]= mapminmax(T);
 T=yf;
 T=T';
 
    for i=1:Runno
        Rrate=0;
        W=Best_pos(i,1:63);
        B=Best_pos(i,64:75);
        for pp=1:150
            actualvalue=my_simulate(4,9,3,W,B,I2(pp,:));
            if(T(pp)==-1)
                if (actualvalue(1)>=0.95 && actualvalue(2)<0.05 && actualvalue(3)<0.05)
                    Rrate=Rrate+1;
                end
            end
            if(T(pp)==0)
                if (actualvalue(1)<0.05 && actualvalue(2)>=0.95 && actualvalue(3)<0.05)
                    Rrate=Rrate+1;
                end  
            end
            if(T(pp)==1)
                if (actualvalue(1)<0.05 && actualvalue(2)<0.05 && actualvalue(3)>=0.95)
                    Rrate=Rrate+1;
                end              
            end
        end
        
        Final_rate(i,:)=(Rrate/150)*100;
    end
    
   ['classification rate = ' , num2str(Final_rate(i))]
    
end
   

if Function_name=='F4'
    
    load Cancer.txt
 x=Cancer;
 %I2=x(1:150,1:4);
 H2=x(1:699,2:11);
 for iii=1:699
     for jjj=1:10
         H2(iii,jjj)=((H2(iii,jjj)-1)/9);
     end
 end
 I2=H2(1:699,1:9);
 
 T=H2(1:699,10);
 Hno=19;
 dim=11*19;
 
    for i=1:Runno
        Rrate=0;
        W=Best_pos(i,1:10*Hno);
        B=Best_pos(i,10*Hno+1:dim);
        for pp=600:699
            actualvalue=my_simulate(9,Hno,1,W,B,I2(pp,:) );
            if(T(pp)>=0.3 && T(pp)<0.4)
                if (abs(actualvalue-0.333333333333333)<0.1)
                    Rrate=Rrate+1;
                end
            end
            if(T(pp)>=0.1 && T(pp)<0.2)
                if (abs(actualvalue-0.111111111111111)<0.1)
                    Rrate=Rrate+1;
                end  
            end

        end
        
        Final_rate(1,i)=(Rrate/100)*100;
    end
    
   ['classification rate = ' , num2str(Final_rate(i))]
end
    
if Function_name=='F5'

load Heart_test.txt
 x=Heart_test;
 %I2=x(1:150,1:4);
 I2(:,1)=x(1:187,2);
 I2(:,2)=x(1:187,3);
 I2(:,3)=x(1:187,4);
 I2(:,4)=x(1:187,5);
 I2(:,5)=x(1:187,6);
 I2(:,6)=x(1:187,7);
 I2(:,7)=x(1:187,8);
 I2(:,8)=x(1:187,9);
 I2(:,9)=x(1:187,10);
 I2(:,10)=x(1:187,11);
 I2(:,11)=x(1:187,12);
 I2(:,12)=x(1:187,13);
 I2(:,13)=x(1:187,14);
 I2(:,14)=x(1:187,15);
 I2(:,15)=x(1:187,16);
 I2(:,16)=x(1:187,17);
 I2(:,17)=x(1:187,18);
 I2(:,18)=x(1:187,19);
 I2(:,19)=x(1:187,20);
 I2(:,20)=x(1:187,21);
 I2(:,21)=x(1:187,22); 
 I2(:,22)=x(1:187,23);  
 T=x(1:187,1);
 %I=(I2-0.1)./(7.9-0.1)
 
Hno=45;
dim = 24*45+1
 
    for i=1:Runno

        Rrate=0;
        W=Best_pos(i,1:23*Hno);
        B=Best_pos(i,23*Hno+1:dim);
        for pp=1:80
            actualvalue=my_simulate(22,Hno,1,W,B,I2(pp,:) );
            if(T(pp)==1)
                if (actualvalue>=0.95)
                    Rrate=Rrate+1;
                end
            end
            if(T(pp)==0)
                if (actualvalue(1)<0.05)
                    Rrate=Rrate+1;
                end  
            end

        end
        
       Final_rate(i)=(Rrate/80)*100;
    end
    
   ['classification rate = ' , num2str(Final_rate(i))]

end

if Function_name=='F6'
    
    Hnode=15;
dim = 3*Hnode+1
 
%for test 3 times more than the training samples
%   xf1=[0:0.01:pi];
%   yf1=sin(2.*xf1);
%   yf1=yf1.*exp(-xf1);
 xf1=[-3:0.05:3];
%  yf1=sin(2.*xf1);
%  yf1=yf1.*exp(-xf1);
 %yf1=xf1.^2;
%yf1=xf1.^4-6.*xf1.^2+3;
yf1=sigmf(xf1,[1 0]);

%   xf1=[-2*pi:0.05:2*pi];
%   yf1=sin(2.*xf1);
  %yf1=yf1.*exp(-xf1);
 yNN=zeros(1,10);
 [xf,PS] = mapminmax(xf1);
 [yf,PS2]= mapminmax(yf1);
  [M N]=size(xf);
  test_error=zeros(1,Runno);
    for i=1:Runno
        for ww=1:2*Hnode
            W(ww)=Best_pos(i,ww);
        end
        for bb=2*Hnode+1:3*Hnode+1
            B(bb-2*Hnode)=Best_pos(i,bb);
        end
        
        for pp=1:N
            yNN(pp)=my_simulate(1,15,1, W,B,xf(pp));
        end
        figure
        set(axes,'FontName','Times New Roman');
        
        hold on
        grid on;
        xfDenorm = mapminmax('reverse',xf,PS); 
        yfDenorm = mapminmax('reverse',yNN,PS2);
        test_error(1,i)=test_error(1,i)+sum(abs( yfDenorm- yf1 ));        
        plot(xf1,yf1,'DisplayName','Real curve','Color','b');
        plot(xfDenorm,yfDenorm,'DisplayName','Approximated curve','Marker','.','LineStyle','-','Color','r');
        %eqtext = '$$sin(2x)e^{-x}$$'; 
        
        name='GWO'

        title([['\fontsize{12}\it ', name]],'FontName','Times New Roman');
        xlabel('\fontsize{12}\it X');ylabel('\fontsize{12}\it Y');
        legend('toggle');
        set(legend,'FontAngle','italic','FontName','Times New Roman')        

  
    end
      
end

   
if Function_name=='F7'
 
Hnode=15;
dim = 3*Hnode+1
 
%for test 3 times more than the training samples
%   xf1=[0:0.01:pi];
%   yf1=sin(2.*xf1);
%   yf1=yf1.*exp(-xf1);
 %xf1=[-3:0.05:3];
%  yf1=sin(2.*xf1);
%  yf1=yf1.*exp(-xf1);
 %yf1=xf1.^2;
%yf1=xf1.^4-6.*xf1.^2+3;

  xf1=[1.25:0.04:2.75];
  yf1=power(cos(xf1.*pi/2),7);
  %yf1=yf1.*exp(-xf1);
 yNN=zeros(1,10);
 [xf,PS] = mapminmax(xf1);
 [yf,PS2]= mapminmax(yf1);
  [M N]=size(xf);
  test_error=zeros(1,Runno);
    for i=1:Runno
        for ww=1:2*Hnode
            W(ww)=Best_pos(i,ww);
        end
        for bb=2*Hnode+1:3*Hnode+1
            B(bb-2*Hnode)=Best_pos(i,bb);
        end
        
        for pp=1:N
            yNN(pp)=my_simulate(1,15,1, W,B,xf(pp));
        end
        figure
        set(axes,'FontName','Times New Roman');
        
        hold on
        grid on;
        xfDenorm = mapminmax('reverse',xf,PS); 
        yfDenorm = mapminmax('reverse',yNN,PS2);
        test_error(1,i)=test_error(1,i)+sum(abs( yfDenorm- yf1 ));        
        plot(xf1,yf1,'DisplayName','Real curve','Color','b');
        plot(xfDenorm,yfDenorm,'DisplayName','Approximated curve','Marker','.','LineStyle','-','Color','r');
        %eqtext = '$$sin(2x)e^{-x}$$'; 
      name='GWO'
        
        
        title([['\fontsize{12}\it ', name]],'FontName','Times New Roman');
        xlabel('\fontsize{12}\it X');ylabel('\fontsize{12}\it Y');
        legend('toggle');
        set(legend,'FontAngle','italic','FontName','Times New Roman')        
    end
   
end

if Function_name=='F8'

Hnode=15;
dim = 3*Hnode+1
 
%for test 3 times more than the training samples
%   xf1=[0:0.01:pi];
%   yf1=sin(2.*xf1);
%   yf1=yf1.*exp(-xf1);
 %xf1=[-3:0.05:3];
%  yf1=sin(2.*xf1);
%  yf1=yf1.*exp(-xf1);
 %yf1=xf1.^2;
%yf1=xf1.^4-6.*xf1.^2+3;

  xf1=[-2*pi:0.05:2*pi];
  yf1=sin(2.*xf1);
  %yf1=yf1.*exp(-xf1);
 yNN=zeros(1,10);
 [xf,PS] = mapminmax(xf1);
 [yf,PS2]= mapminmax(yf1);
  [M N]=size(xf);
  test_error=zeros(1,Runno);
    for i=1:Runno
        for ww=1:2*Hnode
            W(ww)=Best_pos(i,ww);
        end
        for bb=2*Hnode+1:3*Hnode+1
            B(bb-2*Hnode)=Best_pos(i,bb);
        end
        
        for pp=1:N
            yNN(pp)=my_simulate(1,15,1, W,B,xf(pp));
        end
        figure
        set(axes,'FontName','Times New Roman');
        
        hold on
        grid on;
        xfDenorm = mapminmax('reverse',xf,PS); 
        yfDenorm = mapminmax('reverse',yNN,PS2);
        test_error(1,i)=test_error(1,i)+sum(abs( yfDenorm- yf1 ));        
        plot(xf1,yf1,'DisplayName','Real curve','Color','b');
        plot(xfDenorm,yfDenorm,'DisplayName','Approximated curve','Marker','.','LineStyle','-','Color','r');
        %eqtext = '$$sin(2x)e^{-x}$$'; 
        name='GWO'
        
        
        title([['\fontsize{12}\it ', name]],'FontName','Times New Roman');
        xlabel('\fontsize{12}\it X');ylabel('\fontsize{12}\it Y');
        legend('toggle');
        set(legend,'FontAngle','italic','FontName','Times New Roman')        
    end

end


if Function_name=='F9'

Hnode=15;
dim = 4*Hnode+1
 
%for test 3 times more than the training samples
%   xf1=[0:0.01:pi];
%   yf1=sin(2.*xf1);
% %   yf1=yf1.*exp(-xf1);
%  xf1=[-2:0.05:2];
% %  yf1=sin(2.*xf1);
% %  yf1=yf1.*exp(-xf1);
%  yf1=xf1.^2;
% %yf1=xf1.^4-6.*xf1.^2+3;
%yf1=sigmf(xf1,[1 0]);

 xf1=[-2:0.05:2];
 yf1=xf1.^2; 
  [xf1,yf1] = meshgrid(-2:.1:2);
% yf1=xf1.^4-6.*xf1.^2+3;
%yf1=sigmf(xf1,[1 0]);
   [M N]=size(xf1);
for i=1:M
    for j=1:N
        L=[xf1(i,j) yf1(i,j)];
        zf1(i,j)=sum(L.^2);
    end
end

   
 [xf,PS] = mapminmax(xf1);
 [yf,PS2]= mapminmax(yf1);
 [zf,PS3]= mapminmax(zf1);
  
  
  zNN=zeros(1,10);
  test_error=zeros(1,Runno);
    for i=1:Runno
        for ww=1:3*Hnode
            W(ww)=Best_pos(i,ww);
        end
        for bb=3*Hnode+1:4*Hnode+1
            B(bb-3*Hnode)=Best_pos(i,bb);
        end
        
%         for pp=1:N
%             zNN(pp)=my_simulate(W,B,xf(pp),Hnode);
%         end
for ii=1:M
    for jj=1:N
        L=[xf1(ii,jj) yf1(ii,jj)];
         zNN(ii,jj)=my_simulate_2_inputs(W,B,xf1(ii,jj),yf1(i,jj),Hnode);
    end
end
        figure
        set(axes,'FontName','Times New Roman');
        
        hold on
        grid on;
        xfDenorm = mapminmax('reverse',xf,PS); 
        yfDenorm = mapminmax('reverse',yf,PS2);
        zfDenorm = mapminmax('reverse',zNN,PS3);
        test_error(1,i)=test_error(1,i)+sum(sum(abs( zfDenorm- zf1 )));  
        %surfc(xf1,yf1,zf1);
        %colormap('Summer');
        surfc(xfDenorm,yfDenorm,zfDenorm);
        view([-38,30])
        
        %colormap('Winter');
        %plot(xf1,yf1,'DisplayName','Real curve','Color','b');
        %plot(xfDenorm,yfDenorm,'DisplayName','Approximated curve','Marker','.','LineStyle','-','Color','r');
        %eqtext = '$$sin(2x)e^{-x}$$'; 
       
        name='GWO';
  
        
        title([['\fontsize{12}\it ', name]],'FontName','Times New Roman');
        xlabel('\fontsize{12}\it X');ylabel('\fontsize{12}\it Y');zlabel('\fontsize{12}\it Z');
        %legend('toggle');
        set(legend,'FontAngle','italic','FontName','Times New Roman')       


    end
end